#if !defined(__SIMPLEMMAPPEDFILE_H__)
#define	__SIMPLEMMAPPEDFILE_H__

#if defined(_MSC_VER)
#pragma once
#endif

class CSimpleMemoryMappedFileImpl
{
    HANDLE m_hMapFile;
    PBYTE  m_pBaseAddress;
    LONG   m_nFileSize;
    LONG   m_nCurrentPosition;
    bool   m_bError;

public:
    CSimpleMemoryMappedFileImpl() : m_hMapFile( NULL), m_pBaseAddress( NULL), m_nFileSize( 0),
        m_nCurrentPosition( 0), m_bError( false)
    {
    }

    ~CSimpleMemoryMappedFileImpl()
    {
        Close();
    }

    bool Open( const TCHAR * fileName, const TCHAR * mode)
    {
        //
        //        - 
        //  .
        //
        _ASSERTE( m_hMapFile == NULL);

        HANDLE hFile;
        DWORD dwDesiredAccess, dwCreationDisposition, flProtect, dwMapAccess;

        if (_tcscmp( mode, _T("rw")) == 0)
        {
            dwDesiredAccess       = GENERIC_READ | GENERIC_WRITE;
            dwCreationDisposition = OPEN_EXISTING;
            flProtect             = PAGE_READWRITE;
            dwMapAccess           = FILE_MAP_WRITE;
        }
        else if (_tcscmp( mode, _T("r")) == 0  ||  _tcscmp( mode, _T("rb")) == 0)
        {
            dwDesiredAccess       = GENERIC_READ;
            dwCreationDisposition = OPEN_EXISTING;
            flProtect             = PAGE_READONLY;
            dwMapAccess           = FILE_MAP_READ;
        }
        else if (_tcscmp( mode, _T("w")) == 0)
        {
            dwDesiredAccess       = GENERIC_WRITE;
            dwCreationDisposition = OPEN_EXISTING;
            flProtect             = PAGE_READWRITE;
            dwMapAccess           = FILE_MAP_WRITE;
        }
        else
        {
            _ASSERTE( 0);
            return false;
        }

        hFile = ::CreateFile( fileName, dwDesiredAccess, FILE_SHARE_READ, NULL,
            dwCreationDisposition, FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile == INVALID_HANDLE_VALUE)
            return false;

        DWORD dwFileSizeLow, dwFileSizeHigh;
        dwFileSizeLow = ::GetFileSize( hFile, &dwFileSizeHigh);
        if (dwFileSizeLow == INVALID_FILE_SIZE || dwFileSizeLow == 0)
        {
            ::CloseHandle( hFile);
            return false;
        }

        m_hMapFile = ::CreateFileMapping( hFile, NULL, flProtect, dwFileSizeHigh,
            dwFileSizeLow, NULL);

        ::CloseHandle( hFile);

        if (m_hMapFile == NULL)
            return false;

        m_pBaseAddress = (PBYTE) ::MapViewOfFile( m_hMapFile, dwMapAccess, 0, 0, 0);
        if (m_pBaseAddress == NULL)
        {
            ::CloseHandle( m_hMapFile);
            return false;
        }

        m_nFileSize = (LONG) dwFileSizeLow;
        m_nCurrentPosition = 0;
        return true;
    }

    void Close()
    {
        if (m_pBaseAddress != NULL)
        {
            ::UnmapViewOfFile( m_pBaseAddress);
            m_pBaseAddress = NULL;
        }

        if (m_hMapFile != NULL)
        {
            ::CloseHandle( m_hMapFile);
            m_hMapFile = NULL;
        }

        m_nFileSize = 0;
        m_nCurrentPosition = 0;
        m_bError = false;
    }

    bool SeekCur( long offs)
    {
        _ASSERTE( IsValid());

        long newPos = m_nCurrentPosition + offs;
        if (newPos < 0)
        {
            _ASSERTE( newPos >= 0);
            return false;
        }

        if (newPos > m_nFileSize)
        {
            //         .
            _ASSERTE( newPos <= m_nFileSize);
            return false;
        }

        m_nCurrentPosition = newPos;
        return true;
    }

    bool SeekEnd( long offs)
    {
        _ASSERTE(0);
        return false;
    }

    bool SeekBeg( long offs)
    {
        _ASSERTE( IsValid());

        if (offs < 0)
        {
            _ASSERTE( offs >= 0);
            return false;
        }

        if (offs > m_nFileSize)
        {
            //         .
            _ASSERTE( offs <= m_nFileSize);
            return false;
        }

        m_nCurrentPosition = offs;
        return true;
    }

    size_t Read( void * buffer, size_t bufSize)
    {
        _ASSERTE( IsValid());

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, TRUE));

        if (m_nCurrentPosition > m_nFileSize)
            return 0;

        size_t nRead;
        if (m_nCurrentPosition + (LONG) bufSize > m_nFileSize)
            nRead = (size_t)(m_nFileSize - m_nCurrentPosition);
        else
            nRead = bufSize;

        memcpy( buffer, m_pBaseAddress + m_nCurrentPosition, nRead);
        m_nCurrentPosition += (LONG) nRead;
        return nRead;
    }

    size_t Write( const void * buffer, size_t bufSize)
    {
        _ASSERTE( IsValid());

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, FALSE));

        if (m_nCurrentPosition + (LONG) bufSize > m_nFileSize)
        {
            _ASSERTE( m_nCurrentPosition + (LONG) bufSize <= m_nFileSize);
            return 0;
        }

        memcpy( m_pBaseAddress + m_nCurrentPosition, (PBYTE) buffer, bufSize);
        m_nCurrentPosition += (LONG) bufSize;

        return bufSize;
    }

    long Tell()
    {
        return m_nCurrentPosition;
    }

    bool IsEof() const
    {
        return (m_nCurrentPosition >= m_nFileSize);
    }

    bool IsError() const
    {
        return (IsValid() == false || m_bError == true);
    }

    bool operator!() const
    {
        return IsError();
    }

    operator bool() const
    {
        return !IsError();
    }

private:
    bool IsValid() const
    {
        return (m_hMapFile != NULL && m_pBaseAddress != NULL && m_nFileSize != 0);
    }
};

#endif // __SIMPLEMMAPPEDFILE_H__
