#if !defined(__SIMPLEMMAPPEDFILE_H__)
#define	__SIMPLEMMAPPEDFILE_H__

#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>

typedef unsigned int DWORD;
typedef long LONG;
typedef unsigned char *PBYTE;
#define TRUE true
#define FALSE true
#define _ASSERTE(b) assert(b)
#define _CrtIsValidPointer(...) true

class CSimpleMemoryMappedFileImpl
{
    char *m_pBaseAddress;
    long m_nFileSize;
    long m_nCurrentPosition;
    bool m_bError;

public:
    CSimpleMemoryMappedFileImpl() : m_pBaseAddress( NULL), m_nFileSize( 0),
        m_nCurrentPosition( 0), m_bError( false)
    {
    }

    ~CSimpleMemoryMappedFileImpl()
    {
        Close();
    }

    bool Open( const char * fileName, const char * mode)
    {
        //
        //        - 
        //  .
        //
        _ASSERTE( m_pBaseAddress == NULL);

        int hFile, desiredAccess;
        DWORD dwDesiredAccess, dwCreationDisposition, flProtect, dwMapAccess;

        if ( strcmp( mode, "rw") == 0) {
            desiredAccess = O_RDWR;
            flProtect = PROT_READ | PROT_WRITE;
	}
        else if ( strcmp( mode, "r") == 0  || strcmp( mode, "rb") == 0) {
            desiredAccess = O_RDONLY;
            flProtect = PROT_READ;
	}
        else if ( strcmp( mode, "w") == 0) {
            dwDesiredAccess = O_WRONLY;
            flProtect = PROT_WRITE;
	}
        else
        {
            _ASSERTE( 0);
            return false;
        }

	hFile = open( fileName, desiredAccess );
        if (hFile == -1)
            return false;

	DWORD dwFileSize;
	dwFileSize = lseek( hFile, 0, SEEK_END );

        if (dwFileSize == -1 || dwFileSize == 0)
        {
            ::close( hFile);
            return false;
        }

	m_pBaseAddress = (char*) mmap( 0, dwFileSize, flProtect, MAP_SHARED, hFile, 0 );

        ::close( hFile);

        if (m_pBaseAddress == MAP_FAILED)
            return false;

        m_nFileSize = (LONG) dwFileSize;
        m_nCurrentPosition = 0;
        return true;
    }

    void Close()
    {
        if (m_pBaseAddress != NULL)
        {
            munmap( m_pBaseAddress, m_nFileSize);
            m_pBaseAddress = NULL;
        }

        m_nFileSize = 0;
        m_nCurrentPosition = 0;
        m_bError = false;
    }

    bool SeekCur( long offs)
    {
        _ASSERTE( IsValid());

        long newPos = m_nCurrentPosition + offs;
        if (newPos < 0)
        {
            _ASSERTE( newPos >= 0);
            return false;
        }

        if (newPos > m_nFileSize)
        {
            //         .
            _ASSERTE( newPos <= m_nFileSize);
            return false;
        }

        m_nCurrentPosition = newPos;
        return true;
    }

    bool SeekEnd( long offs)
    {
        _ASSERTE(0);
        return false;
    }

    bool SeekBeg( long offs)
    {
        _ASSERTE( IsValid());

        if (offs < 0)
        {
            _ASSERTE( offs >= 0);
            return false;
        }

        if (offs > m_nFileSize)
        {
            //         .
            _ASSERTE( offs <= m_nFileSize);
            return false;
        }

        m_nCurrentPosition = offs;
        return true;
    }

    size_t Read( void * buffer, size_t bufSize)
    {
        _ASSERTE( IsValid());

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, TRUE));

        if (m_nCurrentPosition > m_nFileSize)
            return 0;

        size_t nRead;
        if (m_nCurrentPosition + (LONG) bufSize > m_nFileSize)
            nRead = (size_t)(m_nFileSize - m_nCurrentPosition);
        else
            nRead = bufSize;

        memcpy( buffer, m_pBaseAddress + m_nCurrentPosition, nRead);
        m_nCurrentPosition += (LONG) nRead;
        return nRead;
    }

    size_t Write( const void * buffer, size_t bufSize)
    {
        _ASSERTE( IsValid());

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, FALSE));

        if (m_nCurrentPosition + (LONG) bufSize > m_nFileSize)
        {
            _ASSERTE( m_nCurrentPosition + (LONG) bufSize <= m_nFileSize);
            return 0;
        }

        memcpy( m_pBaseAddress + m_nCurrentPosition, (PBYTE) buffer, bufSize);
        m_nCurrentPosition += (LONG) bufSize;

        return bufSize;
    }

    long Tell()
    {
        return m_nCurrentPosition;
    }

    bool IsEof() const
    {
        return (m_nCurrentPosition >= m_nFileSize);
    }

    bool IsError() const
    {
        return (IsValid() == false || m_bError == true);
    }

    bool operator!() const
    {
        return IsError();
    }

    operator bool() const
    {
        return !IsError();
    }

private:
    bool IsValid() const
    {
        return (m_pBaseAddress != NULL && m_nFileSize != 0);
    }
};

#endif // __SIMPLEMMAPPEDFILE_H__
