#ifndef	__SIMPLESTDIOFILE_H__
#define	__SIMPLESTDIOFILE_H__

#if defined(_MSC_VER)
#pragma once
#endif

#ifdef __linux__
typedef char TCHAR;
typedef unsigned char BYTE;
typedef unsigned short WORD;
#define _tfopen fopen
#endif

class CSimpleStdioFileImpl
{
    FILE *m_hFile;

public:
    CSimpleStdioFileImpl() : m_hFile( NULL)
    {
    }

    ~CSimpleStdioFileImpl()
    {
        Close();
    }

    bool Open( const TCHAR * fileName, const TCHAR * mode)
    {
        _ASSERTE( m_hFile == NULL);
        m_hFile = _tfopen( fileName, mode);
        return (m_hFile != NULL);
    }

    void Close()
    {
        if (m_hFile != NULL)
        {
            fclose( m_hFile);
            m_hFile = NULL;
        }
    }

    bool SeekCur( long offs)
    {
        _ASSERTE( m_hFile != NULL);
        return (fseek( m_hFile, offs, SEEK_CUR) == 0);
    }

    bool SeekEnd( long offs)
    {
        _ASSERTE( m_hFile != NULL);
        return (fseek( m_hFile, offs, SEEK_END) == 0);
    }

    bool SeekBeg( long offs)
    {
        _ASSERTE( m_hFile != NULL);
        return (fseek( m_hFile, offs, SEEK_SET) == 0);
    }

    size_t Read( void * buffer, size_t bufSize)
    {
        _ASSERTE( m_hFile != NULL);

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, TRUE));

        return fread( buffer, 1, bufSize, m_hFile);
    }

    size_t Write( const void * buffer, size_t bufSize)
    {
        _ASSERTE( m_hFile != NULL);

        if (bufSize == 0)
            return 0;

        _ASSERTE( _CrtIsValidPointer( buffer, (unsigned int) bufSize, FALSE));
        return fwrite( buffer, 1, bufSize, m_hFile);
    }

    long Tell()
    {
        _ASSERTE( m_hFile != NULL);
        return ftell( m_hFile);
    }

    bool IsEof() const
    {
        _ASSERTE( m_hFile != NULL);
        return (feof( m_hFile) != 0);
    }

    bool IsError() const
    {
        _ASSERTE( m_hFile != NULL);
        return (ferror( m_hFile) != 0);
    }

    bool operator!() const
    {
        if (m_hFile == NULL)
            return true;
        else
            return IsError();
    }

    operator bool() const
    {
        if (m_hFile == NULL)
            return false;
        else
            return !IsError();
    }

    operator FILE*()
    {
        return m_hFile;
    }
};


#endif // __SIMPLESTDIOFILE_H__
