/* 
 * This file is part of Pingvo.
 *
 * Copyright (C) 2003-2009 Vad Rulezz <vr5@narod.ru>
 *
 * Pingvo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xlibint.h>
#include <X11/extensions/record.h>

Display *rec_dpy, *ctl_dpy;
XRecordContext ctx;
int recorder_datafd, state;
static bool xrec_initialized = false;

void recorder_init() {
  int vmajor, vminor;
  XRecordClientSpec clients[] = { XRecordAllClients };
  XRecordRange *range;

  rec_dpy = XOpenDisplay( 0 );
  if ( !XRecordQueryVersion( rec_dpy, &vmajor, &vminor ) ) return;
  recorder_datafd = ConnectionNumber( rec_dpy );

  ctl_dpy = XOpenDisplay( 0 );
  range = XRecordAllocRange();
  range->device_events.first = 2;
  range->device_events.last = 15;
  range->client_started = true;
  range->client_died = true;
  ctx = XRecordCreateContext( ctl_dpy,
        XRecordFromServerTime | XRecordFromClientTime,
        clients, 1, &range, 1 );
  XFree( range );
  XSync( ctl_dpy, false );

  state = 0;
  xrec_initialized = true;
}

void recorder_cleanup() {
  if ( !xrec_initialized ) return;

  XRecordDisableContext( ctl_dpy, ctx );
  XRecordProcessReplies( rec_dpy );
  XRecordFreeContext( ctl_dpy, ctx );
  XSync( ctl_dpy, false );
  XCloseDisplay( rec_dpy );
  XCloseDisplay( ctl_dpy );
}

union xrcData {
  unsigned char type; 
  xEvent ev;
  xReply reply;
};

void magicReqHandler();

void recording_cb( XPointer /* closure */, XRecordInterceptData *data ) {
  xrcData *pkt = (xrcData*) data->data;
  int key;
  bool press, release;

  if ( data->category != XRecordFromServer ||
       (pkt->type != KeyPress && pkt->type != KeyRelease) ) {
  	XRecordFreeData( data );
	return;
  }

  press = (pkt->type == KeyPress);
  release = !press;

  key = pkt->ev.u.u.detail;
//  printf( "state %d, key: %d, pressed: %s\n", state, key, press? "yes": "no" );

  switch ( state ) {
  case 0:
  	if ( press && (key == 37 || key == 109) ) state = 1; else state = 0;	// Ctrl
	break;
  case 1:
  	if ( press && (key == 90 || key == 106) ) state = 2; else state = 0;	// Ins
	break;
  case 2:
  	if ( release && (key == 90 || key == 106) ) state = 3; else state = 0;
	break;
  case 3:
  	if ( press && (key == 90 || key == 106) ) magicReqHandler();
	state = 0;
	break;
  }

  XRecordFreeData( data );
  return;
}

void do_recording() {
  if ( xrec_initialized )
	XRecordEnableContextAsync( rec_dpy, ctx, recording_cb, 0 );
}

void recorder_process_replies() {
  if ( xrec_initialized )
	XRecordProcessReplies( rec_dpy );
}
